﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Text.RegularExpressions;
using System.Windows.Controls;
using System.Windows;
using System.Windows.Shapes;
using System.Threading;
using System.Windows.Media.Animation;
using System.Windows.Media;
using System.Windows.Threading;

namespace VisualLinq
{
    public abstract class VisualChild<T> : VisualEnumerable<T>
    {
        Line line;
        readonly object padlock = new object();

        protected VisualChild(IVisualEnumerable parent, string text)
            : base(parent.Canvas, CalculateLocation(parent), text)
        {
            line = new Line();
            line.Y1 = Canvas.GetTop(parent.Box) + parent.Box.Height;
            line.X1 = Canvas.GetLeft(parent.Box) + parent.Box.Width / 2;
            line.Y2 = Canvas.GetTop(Box);
            line.X2 = Canvas.GetLeft(Box) + Box.Width / 2;
            line.Stroke = Brushes.Black;
            this.Canvas.Children.Add(line);
        }

        protected void AnimateDataMovement(string text)
        {
            AnimateDataMovement(text, line, 5, 0, 5, -15);
        }

        protected void AnimateDataMovement(string text, Line connection, double x1, double y1, double x2, double y2)
        {
            Animate(() =>
            {
                Storyboard storyboard = new Storyboard();
                Duration duration = new Duration(TimeSpan.FromSeconds(1));

                // FIXME: Work out how to do this nicely!
                DoubleAnimation horizontal = new DoubleAnimation(connection.X1+x1, connection.X2+x2, duration);
                DoubleAnimation vertical = new DoubleAnimation(connection.Y1+y1, connection.Y2+y2, duration);
                storyboard.Children.Add(horizontal);
                storyboard.Children.Add(vertical);
                TextBlock textBlock = new TextBlock();
                textBlock.Text = text;
                textBlock.Name = "dataItem";
                Canvas.RegisterName(textBlock.Name, textBlock);
                Canvas.Children.Add(textBlock);
                Canvas.SetLeft(textBlock, horizontal.From.Value);
                Canvas.SetTop(textBlock, vertical.From.Value);
                Storyboard.SetTargetName(horizontal, "dataItem");
                Storyboard.SetTargetName(vertical, "dataItem");
                Storyboard.SetTargetProperty(horizontal, new PropertyPath(Canvas.LeftProperty));
                Storyboard.SetTargetProperty(vertical, new PropertyPath(Canvas.TopProperty));

                storyboard.Completed += delegate
                {
                    Canvas.Children.Remove(textBlock);
                    Canvas.UnregisterName(textBlock.Name);
                };

                return storyboard;
            });
        }

        static Point CalculateLocation(IVisualEnumerable parent)
        {
            Point p = new Point();
            p.X = Canvas.GetLeft(parent.Box) + parent.Box.Width/2;
            p.Y = Canvas.GetTop(parent.Box) + parent.Box.Height/2 + 100;
            return p;
        }

        protected static string FormatExpression(System.Linq.Expressions.Expression expression)
        {
            string code = expression.ToString();
            Type type = expression.Type;
            if (type.Name.StartsWith("<>")) // Anonymous type
            {
                Regex regex = new Regex(Regex.Escape(type.Name) + @"\((.*)\)");
                code = regex.Replace(code, "{ $1 }");
            }
            return code;
        }

        internal override void ThreadedDispose()
        {
            base.ThreadedDispose();
            Canvas.Children.Remove(line);
        }
    }
}
