﻿using System;
using System.Collections.Generic;
using System.Windows.Controls;
using System.Linq;
using System.Text;
using System.Linq.Expressions;
using System.Windows.Media;
using System.Windows.Shapes;
using System.Windows.Threading;

namespace VisualLinq
{
    public class VisualSelectMany<TResult,TFirst,TSecond> : VisualChild<TResult>
    {
        VisualEnumerable<TFirst> source;
        Func<TFirst,VisualEnumerable<TSecond>> selector;
        Func<TFirst,TSecond,TResult> resultSelector;

        public VisualSelectMany(VisualEnumerable<TFirst> source,
                                Expression<Func<TFirst,VisualEnumerable<TSecond>>> selector,
                                Expression<Func<TFirst,TSecond,TResult>> resultSelector)
            : base(source, FormatName(selector, resultSelector))
        {
            this.source = source;
            this.selector = selector.Compile();
            this.resultSelector = resultSelector.Compile();
        }

        public override IEnumerator<TResult>  GetEnumerator()
        {
            foreach (TFirst element in source)
            {                
                AnimateDataMovement(element.ToString());

                // FIXME: Animate
                using (VisualEnumerable<TSecond> second = CreateOtherSource(element))
                {
                    Line connection = null;

                    Invoke( () =>
                    {
                        connection = new Line
                        {
                            X1 = Canvas.GetLeft(second.Box),
                            Y1 = Canvas.GetTop(second.Box) + second.Box.Height,
                            X2 = Canvas.GetLeft(Box) + Box.Width,
                            Y2 = Canvas.GetTop(Box) + Box.Height/2,
                            Stroke = Brushes.Black
                        };
                        Canvas.Children.Add(connection);
                    });

                    foreach (TSecond secondElement in second)
                    {
                        AnimateDataMovement(secondElement.ToString(), connection, 5, 0, 5, 0);
                        // FIXME: Animate transition
                        yield return resultSelector(element, secondElement);
                    }

                    Canvas.Dispatcher.Invoke(DispatcherPriority.Normal, (Action)(() => Canvas.Children.Remove(connection)));
                }
            }
        }

        VisualEnumerable<TSecond> CreateOtherSource(TFirst element)
        {
            return (VisualEnumerable<TSecond>) Canvas.Dispatcher.Invoke(DispatcherPriority.Normal, (Func<VisualEnumerable<TSecond>>) (() => selector(element)));
        }

        static string FormatName(Expression<Func<TFirst, VisualEnumerable<TSecond>>> selector,
                                 Expression<Func<TFirst, TSecond, TResult>> resultSelector)
        {
            return "stuff";
        }
    }
}
